@@ -305,7 +305,7 @@ class MuData:
             from .io import write_h5mu, read_h5mu
 
             write_h5mu(filename, self)
-            return read_h5mu(filename, self.filemode)
+            return read_h5mu(filename, self.file._filemode)
 
     def strings_to_categoricals(self, df: Optional[pd.DataFrame] = None):
         """
@@ -482,7 +482,7 @@ class MuData:
         data_mod = _restore_index(data_mod)
         data_mod.index.set_names(rowcol, inplace=True)
         data_global.index.set_names(rowcol, inplace=True)
-        for mod in self.mod.keys():
+        for mod, amod in self.mod.items():
             colname = mod + ":" + rowcol
             # use 0 as special value for missing
             # we could use a pandas.array, which has missing values support, but then we get an Exception upon hdf5 write
@@ -492,7 +492,7 @@ class MuData:
             col = col.astype(np.uint32)
             data_mod.loc[:, colname] = col
             data_mod.set_index(colname, append=True, inplace=True)
-            if mod in attrmap:
+            if mod in attrmap and np.sum(attrmap[mod] > 0) == getattr(amod, attr).shape[0]:
                 data_global.set_index(attrmap[mod], append=True, inplace=True)
                 data_global.index.set_names(colname, level=-1, inplace=True)
 
@@ -572,6 +572,14 @@ class MuData:
                 ),
             )
         )
+        # Make sure modname-prefix columns exist in modalities,
+        # keep them in place if they don't
+        for mod in self.mod:
+            for i, col in enumerate(getattr(self, attr).columns):
+                if col.startswith(mod + ":"):
+                    mcol = col[len(mod) + 1 :]
+                    if mcol not in getattr(self.mod[mod], attr).columns:
+                        columns_global[i] = True
         # Only keep data from global .obs/.var columns
         newdf = getattr(self, attr).loc[:, columns_global]
         if inplace:
@@ -828,7 +836,7 @@ class MuData:
     @varp.setter
     def varp(self, value):
         varp = PairwiseArrays(self, 0, vals=convert_to_dict(value))
-        if self.is_vew:
+        if self.is_view:
             self._init_as_actual(self.copy())
         self._varp = varp
 
